@@ -1,12 +1,19 @@
 """
 Provide various plotting utilities for displaying results of e.g. the inference
-or prediction process.
+or prediction process. At the moment, three subcommands are grouped under
+`lyscripts.plot`:
+
+1. `lyscripts.plot.corner`, which simply outputs a corner plot with nice labels for
+a set of drawn samples.
+2. The module `lyscripts.plot.histograms` can be used to draw histograms, e.g. the ones
+over risks and prevalences as computed by the `lyscripts.predict` module.
+3. Module `lyscripts.plot.thermo_int` allows comparing rounds of thermodynamic
+integration for different models.
 """
 import argparse
 from pathlib import Path
 
-from ..helpers import clean_docstring
-from . import __doc__, corner, histograms, thermo_int
+from . import corner, histograms, thermo_int
 
 
 def _add_parser(
@@ -18,8 +25,8 @@ def _add_parser(
     """
     parser = subparsers.add_parser(
         Path(__file__).parent.name,
-        description=clean_docstring(__doc__),
-        help=clean_docstring(__doc__),
+        description=__doc__,
+        help=__doc__,
         formatter_class=help_formatter,
     )
     subparsers = parser.add_subparsers()
