@@ -1,12 +1,18 @@
 """
 This module provides functions and scripts to predict the risk of hidden involvement,
 given observed diagnoses, and prevalences of patterns for diagnostic modalities.
+
+The submodules for prediction are currently:
+
+1. The `lyscripts.predict.prevalences` module for computing prevalences of certain
+involvement patterns that may also be compared to observed prevalences.
+2. A module `lyscripts.predict.risks` for predicting the risk of any specific pattern
+of involvement given any particular diagnosis.
 """
 import argparse
 from pathlib import Path
 
-from ..helpers import clean_docstring
-from . import __doc__, prevalences, risks
+from . import prevalences, risks
 
 
 def _add_parser(
@@ -18,8 +24,8 @@ def _add_parser(
     """
     parser = subparsers.add_parser(
         Path(__file__).parent.name,
-        description=clean_docstring(__doc__),
-        help=clean_docstring(__doc__),
+        description=__doc__,
+        help=__doc__,
         formatter_class=help_formatter,
     )
     subparsers = parser.add_subparsers()
