@@ -1,18 +1,16 @@
 import argparse
 
-from .. import _exit
-from ..helpers import clean_docstring
-from ..rich_argparse import RichHelpFormatter
-from . import __doc__, prevalences, risks
+from lyscripts import RichDefaultHelpFormatter, exit_cli
+from lyscripts.predict import prevalences, risks
 
 # I need another __main__ guard here, because otherwise pdoc tries to run this
 if __name__ == "__main__":
     parser = argparse.ArgumentParser(
-        prog="lyscripts",
-        description=clean_docstring(__doc__),
-        formatter_class=RichHelpFormatter,
+        prog="lyscripts predict",
+        description=__doc__,
+        formatter_class=RichDefaultHelpFormatter,
     )
-    parser.set_defaults(run_main=_exit)
+    parser.set_defaults(run_main=exit_cli)
     subparsers = parser.add_subparsers()
 
     # the individual scripts add `ArgumentParser` instances and their arguments to
