@@ -10,6 +10,7 @@ objective decisions with respect to defining the *elective clinical target volum
 (CTV-N) in radiotherapy.
 """
 import argparse
+import os
 import warnings
 from multiprocessing import Pool
 from pathlib import Path
@@ -19,9 +20,15 @@ import emcee
 import h5py
 import numpy as np
 import pandas as pd
-import yaml
+from numpy.random import MT19937, RandomState, SeedSequence
 
-from .helpers import CustomProgress, clean_docstring, model_from_config, report
+from lyscripts.utils import (
+    CustomProgress,
+    get_modalities_subset,
+    load_yaml_params,
+    model_from_config,
+    report,
+)
 
 
 def _add_parser(
@@ -33,8 +40,8 @@ def _add_parser(
     """
     parser = subparsers.add_parser(
         Path(__file__).name.replace(".py", ""),
-        description=clean_docstring(__doc__),
-        help=clean_docstring(__doc__),
+        description=__doc__,
+        help=__doc__,
         formatter_class=help_formatter,
     )
     _add_arguments(parser)
@@ -58,6 +65,11 @@ def _add_arguments(parser: argparse.ArgumentParser):
         "--params", default="./params.yaml", type=Path,
         help="Path to parameter file"
     )
+    parser.add_argument(
+        "--modalities", nargs="+",
+        default=["max_llh"],
+        help="List of modalities for inference. Must be defined in `params.yaml`"
+    )
     parser.add_argument(
         "--plots", default="./plots", type=Path,
         help="Directory to store plot of acor times",
@@ -66,10 +78,35 @@ def _add_arguments(parser: argparse.ArgumentParser):
         "--ti", action="store_true",
         help="Perform thermodynamic integration"
     )
+    parser.add_argument(
+        "--pools", type=int, required=False,
+        help=(
+            "Number of parallel worker pools (CPU cores) to use. If not provided, it "
+            "will use all cores. If set to zero, multiprocessing will not be used."
+        )
+    )
+    parser.add_argument(
+        "--seed", type=int, default=42,
+        help="Seed value to reproduce the same sampling round."
+    )
 
     parser.set_defaults(run_main=main)
 
 
+class DummyPool:
+    """
+    Dummy class returning `None` instead of a `Pool` instance when the user chose not
+    to use multiprocessing.
+    """
+    _processes = "no parallel"
+
+    def __enter__(self):
+        return None
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        pass
+
+
 class ConvenienceSampler(emcee.EnsembleSampler):
     """Class that adds some useful defaults to the `emcee.EnsembleSampler`."""
     def __init__(
@@ -79,9 +116,12 @@ class ConvenienceSampler(emcee.EnsembleSampler):
         log_prob_fn,
         pool=None,
         backend=None,
+        random_state=None,
     ):
         """Initialize sampler with sane defaults."""
         moves = [(emcee.moves.DEMove(), 0.8), (emcee.moves.DESnookerMove(), 0.2)]
+        if random_state is not None:
+            self.random_state = random_state.get_state()
         super().__init__(nwalkers, ndim, log_prob_fn, pool, moves, backend=backend)
 
     def run_sampling(
@@ -208,7 +248,9 @@ def run_mcmc_with_burnin(
     burnin: Optional[int] = None,
     keep_burnin: bool = False,
     thin_by: int = 1,
+    npools: Optional[int] = None,
     verbose: bool = True,
+    seed: int = 42,
 ) -> Dict[str, Any]:
     """
     Draw samples from the `log_prob_fn` using the `ConvenienceSampler` (subclass of
@@ -227,46 +269,64 @@ def run_mcmc_with_burnin(
 
     If `verbose` is set to `True`, the progress will be displayed.
 
+    The `seed` value is supposed to make the sampling procedure reproducible, but it
+    does not seem to work here.
+
     Returns a dictionary with some information about the burnin phase.
     """
     if sampling_kwargs is None:
         sampling_kwargs = {}
 
-    with Pool() as pool:
+    if npools is None:
+        created_pool = Pool(os.cpu_count())
+    elif npools == 0:
+        created_pool = DummyPool()
+    elif 0 < npools:
+        created_pool = Pool(np.minimum(npools, os.cpu_count()))
+    else:
+        raise ValueError(
+            "Number of pools must be integer larger or equal to 0 (or `None`)"
+        )
+
+    with created_pool as pool:
         # Burnin phase
         if keep_burnin:
             burnin_backend = persistent_backend
         else:
             burnin_backend = emcee.backends.Backend()
 
+        random_state = RandomState(MT19937(SeedSequence(seed)))  # does not work...
         burnin_sampler = ConvenienceSampler(
             nwalkers, ndim, log_prob_fn,
             pool=pool, backend=burnin_backend,
+            random_state=random_state,
         )
 
         if burnin is None:
             burnin_result = burnin_sampler.run_sampling(
-                progress_desc="Burn-in " if verbose else None,
+                progress_desc=f"Burn-in ({created_pool._processes} cores)" if verbose else None,
                 **sampling_kwargs,
             )
         else:
             burnin_result = burnin_sampler.run_sampling(
                 min_steps=burnin,
                 max_steps=burnin,
-                progress_desc="Burn-in " if verbose else None,
+                progress_desc=f"Burn-in ({created_pool._processes} cores)" if verbose else None,
             )
 
         # persistent sampling phase
+        random_state = RandomState(MT19937(SeedSequence(seed)))  # does not work...
         sampler = ConvenienceSampler(
             nwalkers, ndim, log_prob_fn,
             pool=pool, backend=persistent_backend,
+            random_state=random_state,
         )
         sampler.run_sampling(
             min_steps=nsteps,
             max_steps=nsteps,
             thin_by=thin_by,
             initial_state=burnin_result["final_state"],
-            progress_desc="Sampling" if verbose else None,
+            progress_desc=f"Sampling ({created_pool._processes} cores)" if verbose else None,
         )
 
         return burnin_result
@@ -299,39 +359,46 @@ def main(args: argparse.Namespace):
     sampling procedures is stored as well (acceptance rates of samples, estimates
     of the chains' autocorrelation times and the accuracy during the runs).
 
-    Th help via `python -m lyscripts sample --help` shows this output:
+    Th help via `lyscripts sample --help` shows this output:
 
     ```
-    usage: lyscripts sample [-h] [--params PARAMS] [--plots PLOTS] [--ti] input output
+    USAGE: lyscripts sample [-h] [--params PARAMS]
+                            [--modalities MODALITIES [MODALITIES ...]] [--plots PLOTS]
+                            [--ti] [--pools POOLS] [--seed SEED]
+                            input output
 
-    Learn the spread probabilities of the HMM for lymphatic tumor progression using the
-    preprocessed data as input and MCMC as sampling method.
+    Learn the spread probabilities of the HMM for lymphatic tumor progression using
+    the preprocessed data as input and MCMC as sampling method.
 
-    This is the central script performing for our project on modelling lymphatic spread
-    in head & neck cancer. We use it for model comparison via the thermodynamic
+    This is the central script performing for our project on modelling lymphatic
+    spread in head & neck cancer. We use it for model comparison via the thermodynamic
     integration functionality and use the sampled parameter estimates for risk
     predictions. This risk estimate may in turn some day guide clinicians to make more
-    objective decisions with respect to defining the _elective clinical target volume_
+    objective decisions with respect to defining the *elective clinical target volume*
     (CTV-N) in radiotherapy.
 
+    POSITIONAL ARGUMENTS:
+      input                 Path to training data files
+      output                Path to the HDF5 file to store the results in
 
-    POSITIONAL ARGUMENTS
-    input            Path to training data files
-    output           Path to the HDF5 file to store the results in
-
-    OPTIONAL ARGUMENTS
-    -h, --help       show this help message and exit
-    --params PARAMS  Path to parameter file (default: ./params.yaml)
-    --plots PLOTS    Directory to store plot of acor times (default: ./plots)
-    --ti             Perform thermodynamic integration (default: False)
+    OPTIONAL ARGUMENTS:
+      -h, --help            show this help message and exit
+      --params PARAMS       Path to parameter file (default: ./params.yaml)
+      --modalities MODALITIES [MODALITIES ...]
+                            List of modalities for inference. Must be defined in
+                            `params.yaml` (default: ['max_llh'])
+      --plots PLOTS         Directory to store plot of acor times (default: ./plots)
+      --ti                  Perform thermodynamic integration (default: False)
+      --pools POOLS         Number of parallel worker pools (CPU cores) to use. If not
+                            provided, it will use all cores. If set to zero,
+                            multiprocessing will not be used. (default: None)
+      --seed SEED           Seed value to reproduce the same sampling round. (default:
+                            42)
     ```
 
     [^1]: https://doi.org/10.1007/s11571-021-09696-9
     """
-    with report.status("Read in parameters..."):
-        with open(args.params, mode='r') as params_file:
-            params = yaml.safe_load(params_file)
-        report.success(f"Read in params from {args.params}")
+    params = load_yaml_params(args.params)
 
     with report.status("Read in training data..."):
         # Only read in two header rows when using the Unilateral model
@@ -345,11 +412,15 @@ def main(args: argparse.Namespace):
         MODEL = model_from_config(
             graph_params=params["graph"],
             model_params=params["model"],
-            modalities_params=params["modalities"],
+            modalities_params=get_modalities_subset(
+                defined_modalities=params["modalities"],
+                selection=args.modalities,
+            ),
         )
         MODEL.patient_data = inference_data
         ndim = len(MODEL.spread_probs) + MODEL.diag_time_dists.num_parametric
         nwalkers = ndim * params["sampling"]["walkers_per_dim"]
+        thin_by = params["sampling"]["thin_by"]
         report.success(
             f"Set up {type(MODEL)} model with {ndim} parameters and loaded "
             f"{len(inference_data)} patients"
@@ -365,7 +436,6 @@ def main(args: argparse.Namespace):
             temp_schedule = params["sampling"]["temp_schedule"]
             nsteps = params["sampling"]["nsteps"]
             burnin = params["sampling"]["burnin"]
-            thin_by = params["sampling"]["thin_by"]
             report.success("Prepared thermodynamic integration.")
 
         # record some information about each burnin phase
@@ -387,7 +457,9 @@ def main(args: argparse.Namespace):
                 burnin=burnin,
                 keep_burnin=False,
                 thin_by=thin_by,
-                verbose=True
+                verbose=True,
+                npools=args.pools,
+                seed=args.seed,
             )
             plots["acor_times"].append(burnin_info["acor_times"][-1])
             plots["accept_rates"].append(burnin_info["accept_rates"][-1])
@@ -403,10 +475,6 @@ def main(args: argparse.Namespace):
             # make sure path to output file exists
             args.output.parent.mkdir(parents=True, exist_ok=True)
 
-            # set up sampling params
-            ndim = len(MODEL.spread_probs) + MODEL.diag_time_dists.num_parametric
-            nwalkers = ndim * params["sampling"]["walkers_per_dim"]
-
             # prepare backend
             hdf5_backend = emcee.backends.HDFBackend(args.output, name="mcmc")
 
@@ -417,7 +485,10 @@ def main(args: argparse.Namespace):
             nsteps=params["sampling"]["nsteps"],
             persistent_backend=hdf5_backend,
             sampling_kwargs=params["sampling"]["kwargs"],
+            thin_by=thin_by,
             verbose=True,
+            npools=args.pools,
+            seed=args.seed,
         )
         x_axis = np.array(burnin_info["iterations"])
         plots = {
